﻿// Solution created by Boyan Mihaylov
// http://code.dortikum.net

using System;
using System.Collections.Generic;
using System.ComponentModel.Composition;
using System.ComponentModel.Composition.Hosting;
using Ploeh.Samples.MeffyXmas.MenuModel;

namespace BoyanMihaylov.Solution
{
	public sealed class ContainerBuilder
	{
		private readonly Dictionary<Type, CreationPolicy> _types;

		public ContainerBuilder()
		{
			_types = new Dictionary<Type, CreationPolicy>();
			_types.Add(typeof(OliveOil), CreationPolicy.Any);
			_types.Add(typeof(Mayonnaise), CreationPolicy.Any);
			_types.Add(typeof(EggYolk), CreationPolicy.Any);
		}

		public ContainerBuilder WithNonSharedMayonnaise()
		{
			_types[typeof(Mayonnaise)] = CreationPolicy.NonShared;
			return this;
		}

		public ContainerBuilder WithNonSharedEggYolk()
		{
			_types[typeof(EggYolk)] = CreationPolicy.NonShared;
			return this;
		}

		public ContainerBuilder WithNonSharedOil()
		{
			_types[typeof(OliveOil)] = CreationPolicy.NonShared;
			return this;
		}

		public CompositionContainer Build()
		{
			var catalog = new FluentCatalog();
			foreach (var pair in _types)
			{
				catalog.Register(pair.Key).WithPolicy(pair.Value).AsSelf();
			}
			return new CompositionContainer(catalog);
		}
	}

}
