﻿// Solution created by Boyan Mihaylov
// http://code.dortikum.net

using System;
using System.Collections.Generic;
using System.ComponentModel.Composition.Primitives;
using System.Linq;

namespace BoyanMihaylov.Solution
{
	public sealed class FluentCatalog : ComposablePartCatalog, IFluentBuilder
	{
		private readonly object _locker;
		private IQueryable<ComposablePartDefinition> _queryableParts;
		private Dictionary<string, ComposablePartDefinition> _definitions;
		private bool _isDisposed;

		public FluentCatalog()
		{
			_locker = new object();
			_definitions = new Dictionary<string, ComposablePartDefinition>();
		}

		/// <summary>
		/// Registers the specified type.
		/// </summary>
		/// <param name="type">The type.</param>
		/// <returns>The same instance.</returns>
		public IExport Register(Type type)
		{
			return new FluentComposablePartDefinition(type, type.GetContractName(), type.GetTypeIdentity(), UpdatePartDefinition);
		}

		public override IQueryable<ComposablePartDefinition> Parts
		{
			get
			{
				ThrowIfDisposed();

				if (_queryableParts == null)
				{
					lock (_locker)
					{
						if (_queryableParts == null)
						{
							var definitions = new List<ComposablePartDefinition>();
							foreach (KeyValuePair<string, ComposablePartDefinition> pair in _definitions)
							{
								definitions.Add(pair.Value);
							}
							_queryableParts = definitions.AsQueryable();
						}
					}
				}
				return _queryableParts;
			}
		}

		protected override void Dispose(bool disposing)
		{
			if (disposing)
			{
				_isDisposed = true;
			}
			base.Dispose(disposing);
		}

		/// <summary>
		/// Adds the part definition to the list.
		/// </summary>
		/// <param name="contractName">Name of the contract.</param>
		/// <param name="partDefinition">The part definition.</param>
		private void UpdatePartDefinition(string contractName, ComposablePartDefinition partDefinition)
		{
			_definitions[contractName] = partDefinition;
		}

		private void ThrowIfDisposed()
		{
			if (_isDisposed)
				throw new ObjectDisposedException(GetType().ToString());
		}
	}
}
