﻿// Solution created by Boyan Mihaylov
// http://code.dortikum.net

using System;
using System.Collections.Generic;
using System.ComponentModel.Composition.Primitives;
using System.Linq;

namespace BoyanMihaylov.Solution
{
    public sealed class FluentComposablePart : ComposablePart
    {
        private readonly ComposablePartDefinition _definition;
        private readonly Dictionary<ImportDefinition, object> _importValues;
		private Lazy<object> _lazyValue;

        public FluentComposablePart(Type type, ComposablePartDefinition definition)
        {
			if (type == null)
				throw new ArgumentNullException("type");

			if (definition == null)
				throw new ArgumentNullException("definition");

            _definition = definition;
            _importValues = new Dictionary<ImportDefinition, object>();
			_lazyValue = new Lazy<object>(() => CreateInstance(type));
        }

        public override IEnumerable<ExportDefinition> ExportDefinitions
        {
            get { return _definition.ExportDefinitions; }
        }

        public override IEnumerable<ImportDefinition> ImportDefinitions
        {
            get { return _definition.ImportDefinitions; }
        }

        public override IDictionary<string, object> Metadata
        {
            get { return _definition.Metadata; }
        }

        public override object GetExportedValue(ExportDefinition definition)
        {
			if (definition != ExportDefinitions.First()) 
				throw new InvalidOperationException();

			return _lazyValue.Value;
        }

        public override void SetImport(ImportDefinition definition, IEnumerable<Export> exports)
        {
            // here we make assumption that there are exports
            _importValues[definition] = exports.First().Value;
        }

        /// <summary>
        /// Create an instance of a type.
        /// </summary>
        /// <param name="type">The type.</param>
        /// <returns>A new instance of that type.</returns>
		private object CreateInstance(Type type)
		{
            // we assume that if there are imports for this type, 
            // it means they will be imported to its constructor
			object[] parameters = new object[_importValues.Count];
			int index = 0;
			foreach (object importValue in _importValues.Values)
			{
                parameters[index++] = importValue;
			}

			return Activator.CreateInstance(type, parameters);
		}
    }
}
